@@ -1,23 +1,23 @@
 /* pcomplete.h - structure definitions and other stuff for programmable
-		 completion. */
+n		 completion. */
 
-/* Copyright (C) 1999-2002 Free Software Foundation, Inc.
+/* Copyright (C) 1999-2009 Free Software Foundation, Inc.
 
    This file is part of GNU Bash, the Bourne Again SHell.
 
-   Bash is free software; you can redistribute it and/or modify it under
-   the terms of the GNU General Public License as published by the Free
-   Software Foundation; either version 2, or (at your option) any later
-   version.
+   Bash is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
-   Bash is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or
-   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-   for more details.
+   Bash is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License along
-   with Bash; see the file COPYING.  If not, write to the Free Software
-   Foundation, 59 Temple Place, Suite 330, Boston, MA 02111 USA. */
+   You should have received a copy of the GNU General Public License
+   along with Bash.  If not, see <http://www.gnu.org/licenses/>.
+*/
 
 #if !defined (_PCOMPLETE_H_)
 #  define _PCOMPLETE_H_
@@ -124,6 +124,9 @@ extern ITEMLIST it_stopped;
 extern ITEMLIST it_users;
 extern ITEMLIST it_variables;
 
+extern COMPSPEC *pcomp_curcs;
+extern const char *pcomp_curcmd;
+
 /* Functions from pcomplib.c */
 extern COMPSPEC *compspec_create __P((void));
 extern void compspec_dispose __P((COMPSPEC *));
@@ -150,4 +153,6 @@ extern STRINGLIST *completions_to_stringlist __P((char **));
 extern STRINGLIST *gen_compspec_completions __P((COMPSPEC *, const char *, const char *, int, int));
 extern char **programmable_completions __P((const char *, const char *, int, int, int *));
 
+extern void pcomp_set_readline_variables __P((int, int));
+extern void pcomp_set_compspec_options __P((COMPSPEC *, int, int));
 #endif /* _PCOMPLETE_H_ */
